function [tau_d, gamma_hat_dot, log] = AdaptiveControlBase(eta,nu,...
    eta_d,zeta_ref,eta_dotdotd,gamma_hat,R,T)

% Uses non-regressor based adaptive control algorithm to calculate the 
% thrust and torque to put on the base link.

%% Control constants and variables
sigma = 10;
f = [3 3 3 1 1]';
k = 3*ones(6,1);

zeta_ref_ine = zeros(6,1);
zeta_ref_ine(1:3) = R(:,:,1)*zeta_ref(1:3);
zeta_ref_ine(4:6) = T*zeta_ref(4:6);
eta_dot_1 = R(:,:,1)*nu(1:3);
eta_dot_2 = T*nu(4:6);
eta_dot = [eta_dot_1; eta_dot_2];
eta_tilde = eta_d - eta;
eta_tilde_dot = zeta_ref_ine(1:6) - eta_dot;
s = eta_tilde_dot + sigma*eta_tilde;

epsilon = [1 1 1 0.2 0.2 0.2]';

s = epsilon.*s;

mu = [0.5 0.5 2.0 0.1 0.1]';
varepsilon = [0.05 0.05 0.05 0.02 0.02]';

delta = 10;

phi_1 = eta_dotdotd; %zeros(6,1);
phi_2 = eta_dot;
phi_3 = k;
phi_4 = eta_tilde_dot;
phi_5 = eta_tilde;
K_1 = gamma_hat(1)*s*phi_1'/max((norm(s)*norm(phi_1)),delta);
K_2 = gamma_hat(2)*s*phi_2'/max((norm(s)*norm(phi_2)),delta);
K_3 = gamma_hat(3)*s*phi_3'/max((norm(s)*norm(phi_3)),delta);
K_4 = gamma_hat(4)*s*phi_4'/max((norm(s)*norm(phi_4)),delta);
K_5 = gamma_hat(5)*s*phi_5'/max((norm(s)*norm(phi_5)),delta);
    
tau_d = K_1*phi_1 + K_2*phi_2 + K_3*phi_3 + K_4*phi_4 + K_5*phi_5;

%Transforming tau_d to bodyframe
tau_d(1:3) = R(:,:,1)'*tau_d(1:3);
tau_d(4:6) = pinv(T)*tau_d(4:6);

sat = abs(tau_d) > 100;
tau_d(sat) = 100*sign(tau_d(sat));

%% Logging values to be plotted
%Sending values out to be plotted.
log.phi_1 = phi_1;
log.phi_2 = phi_2;
log.phi_3 = phi_3;
log.phi_4 = phi_4;
log.phi_5 = phi_5;

log.K_1 = K_1;
log.K_2 = K_2;
log.K_3 = K_3;
log.K_4 = K_4;
log.K_5 = K_5;

%% Adaptive update
gamma_hat_dot = zeros(5,1);

if (norm(s)*norm(phi_1) < mu(1)) 
    gamma_hat_dot(1)=-varepsilon(1)*gamma_hat(1)+f(1)*norm(s)*norm(phi_1);
else
    gamma_hat_dot(1)=f(1)*norm(s)*norm(phi_1);
end

if (norm(s)*norm(phi_2) < mu(2)) 
    gamma_hat_dot(2)=-varepsilon(2)*gamma_hat(2)+f(2)*norm(s)*norm(phi_2);
else
    gamma_hat_dot(2) = f(2)*norm(s)*norm(phi_2);
end

if (norm(s)*norm(phi_3) < mu(3))
    gamma_hat_dot(3)=-varepsilon(3)*gamma_hat(3)+f(3)*norm(s)*norm(phi_3);
else
    gamma_hat_dot(3) = f(3)*norm(s)*norm(phi_3);
end

if (norm(s)*norm(phi_4) < mu(4)) 
    gamma_hat_dot(4)=-varepsilon(4)*gamma_hat(4)+f(4)*norm(s)*norm(phi_4);
else
    gamma_hat_dot(4) = f(4)*norm(s)*norm(phi_4);
end

if (norm(s)*norm(phi_5) < mu(5))
    gamma_hat_dot(5)=-varepsilon(5)*gamma_hat(5)+f(5)*norm(s)*norm(phi_5);
else
    gamma_hat_dot(5) = f(5)*norm(s)*norm(phi_5);
end

end

