function [joint_torque, gamma_hat_dot_joint, log] = ...
    AdaptiveControlJoint(x,snake, theta_d, theta_dotd, theta_dotdotd,...
    gamma_hat_joint)

% Uses non-regressor based adaptive control algorithm to calculate the 
% torque to put on the joint angles.

theta   = x(8 : 6+snake.n);               
zeta    = x(7+snake.n : 11+2*snake.n);


%% Control constants and variables
sigma_joint = 10;   
f = [10 10 10 3 3]';
k = 3*ones(length(snake.joint_vec),1); 

theta_tilde = theta_d - theta;
theta_tilde_dot = theta_dotd - zeta(7:snake.n+5);
s = theta_tilde_dot + sigma_joint*theta_tilde;

epsilon = [.5 .25 .125 .0625 .03125]'; 
s =epsilon.*s;

mu = [0.4 0.4 2 0.2 0.2]';
varepsilon = [0.05 0.05 0.05 0.02 0.02]';
delta = 10;

phi_1 = theta_dotdotd;
phi_2 = zeta(7:snake.n+5);
phi_3 = k;
phi_4 = theta_tilde_dot;
phi_5 = theta_tilde;

K_1_joint = gamma_hat_joint(1)*s*phi_1'/max((norm(s)*norm(phi_1)),delta);
K_2_joint = gamma_hat_joint(2)*s*phi_2'/max((norm(s)*norm(phi_2)),delta);
K_3_joint = gamma_hat_joint(3)*s*phi_3'/max((norm(s)*norm(phi_3)),delta);
K_4_joint = gamma_hat_joint(4)*s*phi_4'/max((norm(s)*norm(phi_4)),delta);
K_5_joint = gamma_hat_joint(5)*s*phi_5'/max((norm(s)*norm(phi_5)),delta);

joint_torque = K_1_joint*phi_1 + K_2_joint*phi_2 + K_3_joint*phi_3...
    + K_4_joint*phi_4 + K_5_joint*phi_5;

%Sending values out to be plotted.
log.phi_1_joint = phi_1;
log.phi_2_joint = phi_2;
log.phi_3_joint = phi_3;
log.phi_4_joint = phi_4;
log.phi_5_joint = phi_5;

log.K_1_joint = K_1_joint;
log.K_2_joint = K_2_joint;
log.K_3_joint = K_3_joint;
log.K_4_joint = K_4_joint;
log.K_5_joint = K_5_joint;

% Saturating 
sat = abs(joint_torque) > 50;
joint_torque(sat) = 50*sign(joint_torque(sat));

%Adaptive update
gamma_hat_dot_joint = zeros(5,1);

if (norm(s)*norm(phi_1) < mu(1)) 
    gamma_hat_dot_joint(1) = -varepsilon(1)*gamma_hat_joint(1) +...
        f(1)*norm(s)*norm(phi_1);
else
    gamma_hat_dot_joint(1) = f(1)*norm(s)*norm(phi_1);
end

if (norm(s)*norm(phi_2) < mu(2)) 
    gamma_hat_dot_joint(2) = -varepsilon(2)*gamma_hat_joint(2) +...
        f(2)*norm(s)*norm(phi_2);
else
    gamma_hat_dot_joint(2) = f(2)*norm(s)*norm(phi_2);
end

if (norm(s)*norm(phi_3) < mu(3))
    gamma_hat_dot_joint(3) = -varepsilon(3)*gamma_hat_joint(3) +...
        f(3)*norm(s)*norm(phi_3);
else
    gamma_hat_dot_joint(3) = f(3)*norm(s)*norm(phi_3);
end

if (norm(s)*norm(phi_4) < mu(4)) 
    gamma_hat_dot_joint(4) = -varepsilon(4)*gamma_hat_joint(4) +...
        f(4)*norm(s)*norm(phi_4);
else
    gamma_hat_dot_joint(4) = f(4)*norm(s)*norm(phi_4);
end

if (norm(s)*norm(phi_5) < mu(5))
    gamma_hat_dot_joint(5) = -varepsilon(5)*gamma_hat_joint(5) +...
        f(5)*norm(s)*norm(phi_5);
else
    gamma_hat_dot_joint(5) = f(5)*norm(s)*norm(phi_5);
end

end

