function output = RK4solver(h,Tmax,x_0,snake,pp)
% RK4 Function to go through the simulation step for step with Runge Kutta
% 4th order method. 


%% Number of steps
n_steps = Tmax/h;
T_log = linspace(0,Tmax,n_steps+1);

%% Preparing output
x_log = zeros(26+3*snake.n,n_steps+1);
tau_d_log = zeros(6,n_steps+1);
joint_torque_log = zeros(snake.n-1,n_steps+1);
x_ref_log = zeros(20+4*snake.n,n_steps+1);
orientation_log = zeros(3,n_steps+1);
eta_ee_log = zeros(6,n_steps+1);
% eta_ee_ref_log = zeros(6,n_steps+1);
theta_dotd_log = zeros(snake.n-1,n_steps+1);
eta_ee_task_log = zeros(6,n_steps+1);
eta_base_task_log = zeros(6,n_steps+1);
nominal_task_log = zeros(snake.n-1,n_steps+1);

joint_controller_log.phi_1 = zeros(snake.n-1,n_steps+1);
joint_controller_log.phi_2 = zeros(snake.n-1,n_steps+1);
joint_controller_log.phi_3 = zeros(snake.n-1,n_steps+1);
joint_controller_log.phi_4 = zeros(snake.n-1,n_steps+1);
joint_controller_log.phi_5 = zeros(snake.n-1,n_steps+1);


%% Initial state
x_log(:,1) = x_0;

%% Simulation
for i = 1:n_steps
    x = x_log(:,i);
    
    %% Get states
    pos       =  x(1:3);
    quat      =  x(4:7);
    theta     =  x(8 : 6+snake.n); 
    zeta      =  x(7+snake.n : 11+2*snake.n);
    nu        =  zeta(1:6);
    
    %Parameters from updating factor, gamma, in the adaptive 
    %control law for the whole body and for joints.
    gamma_hat = x(11+2*snake.n+1 : 11+2*snake.n+5);
    gamma_hat_link = x(11+2*snake.n+6 : 11+2*snake.n+10);
    
    %Desired positions for base link and joints
    eta_d = x(12+2*snake.n+10 : 17+2*snake.n + 10);
    theta_d = x(18+2*snake.n+10 : 16+3*snake.n + 10);
    
    %% Compute rotation matrices and jacobians
    [R, J]    =  comp_rot_jac(quat,theta,snake);
    
    %% Test path
%     time = i*h;
    %[zeta_ref, eta_ee] = generateTestPath(snake,x,R,time);
    
    %% Find transformation matrix
    eulerAngles = getEulerAngles(R(:,:,1));
    T = getTransformationMatrix(eulerAngles);
    %% Solve inverse kinematics
    [zeta_ref, eta_ee, logTasks, pp.switch_weight] ...
        = inverseKinematics(snake,x,R,J,pp.switch_weight);
    
    zeta_ref_ine = zeta_ref;
    zeta_ref_ine(1:3) = R(:,:,1)*zeta_ref(1:3); 
    zeta_ref_ine(4:6) = T*zeta_ref(4:6);
    zeta_ref_ine_prev = x_ref_log(16+3*snake.n:20+4*snake.n,i);
    
    acc_ref   = calcAcceleration(zeta_ref_ine_prev,zeta_ref_ine,h);
    %acc_ref = zeros(length(zeta_ref),1);
    eta_dotdotd = zeros(6,1);%acc_ref(1:6);
    theta_dotdotd = acc_ref(7:5+snake.n);
    
    %% Transforming to inertial frame, enabling integration
    eta = [pos; eulerAngles];
    eta_dot = zeros(6,1);
    eta_dot(1:3) = R(:,:,1)*nu(1:3);
    eta_dot(4:6) = T*nu(4:6);
    velocity_ine_d = R(:,:,1)*zeta_ref(1:3);
    angvelocity_ine_d = T*zeta_ref(4:6);
    theta_dotd = zeta_ref(7:5+snake.n);
    
    %% Non-regressor based adaptive control
    
    [tau_d,gamma_hat_dot, log_base_con] = AdaptiveControlBase(eta,...
        eta_dot,eta_d,zeta_ref,eta_dotdotd,gamma_hat,R,T);
    [joint_torque, gamma_hat_dot_joint, log_joint_con] = ...
        AdaptiveControlJoint(x,snake, theta_d, theta_dotd,...
        theta_dotdotd, gamma_hat_link);
    
    update_param.gamma_hat_dot = gamma_hat_dot;
    update_param.gamma_hat_dot_link = gamma_hat_dot_joint;
    update_param.velocity_ine_d = velocity_ine_d;
    update_param.angvelocity_ine_d = angvelocity_ine_d;
    update_param.theta_dotd = theta_dotd;
    
    %% Setting thrusters to zero
    thrust = zeros(snake.n_thruster_tot,1);
    
    %% Logging values
    x_ref_log(:,i+1)        = [eta_d;theta_d;zeta_ref;acc_ref; ...
        zeta_ref_ine];
    tau_d_log(:,i+1)        = tau_d;
    joint_torque_log(:,i+1) = joint_torque;
    orientation_log(:,i+1)  = eulerAngles;
    eta_ee_log(:,i+1)       = eta_ee;
    eta_ee_task_log(:,i+1)   = logTasks.ee;
    eta_base_task_log(:,i+1) = logTasks.base;
    nominal_task_log(:,i+1) = logTasks.nominal;
    theta_dotd_log(:,i+1)   = theta_dotd;
    
    joint_controller_log.phi_1(:,i+1) = log_joint_con.phi_1_joint;
    joint_controller_log.phi_2(:,i+1) = log_joint_con.phi_2_joint;
    joint_controller_log.phi_3(:,i+1) = log_joint_con.phi_3_joint;
    joint_controller_log.phi_4(:,i+1) = log_joint_con.phi_4_joint;
    joint_controller_log.phi_5(:,i+1) = log_joint_con.phi_5_joint;


    %% Runge-Kutta 4
    k1 = f_dyn(x,thrust,tau_d,joint_torque,snake,pp,update_param);
    
    x2 = (x+h*k1/2);
    x2(4:7) = normQ(x2(4:7)); %Normalizing the quaternion vector
    k2 = f_dyn(x2,thrust,tau_d,joint_torque,snake,pp,update_param);
    
    x3 = (x+h*k2/2);
    x3(4:7) = normQ(x3(4:7)); %Normalizing the quaternion vector
    k3 = f_dyn(x3,thrust,tau_d,joint_torque,snake,pp,update_param);
    
    x4 = (x+h*k3);
    x4(4:7) = normQ(x4(4:7)); %Normalizing the quaternion vector
    k4 = f_dyn(x4,thrust,tau_d,joint_torque,snake,pp,update_param);
    
    x_final = x + h*(k1 + 2*k2 + 2*k3 + k4)/6;
    x_final(4:7) = normQ(x_final(4:7)); %Normalizing the quaternion vector
    
    %% Saturating adaptive integrators and angles
    x_final = saturateStates(x_final,snake);
    
    x_log(:,i+1) = x_final;
    disp(['Time elapsed ', num2str(i*h)])
    
end
%% Organizing all output values in one struct to simplify
output.x_log = x_log;
output.tau_d_log = tau_d_log;
output.joint_torque_log = joint_torque_log;
output.x_ref_log = x_ref_log;
output.orientation_log = orientation_log;
output.T_log = T_log;
output.eta_ee_log = eta_ee_log;
output.eta_ee_ref_log = eta_ee_task_log;
output.eta_base_task_log = eta_base_task_log;
output.nominal_task_log = nominal_task_log;
output.theta_dotd_log = theta_dotd_log;
output.joint_controller_log = joint_controller_log;

end
    
















