%This is a simple smiulator for an underwater snake-like robot. The code is
%not optimized for speed and there may still be bugs. Please report back to
%me if you find anything fishy :)
%
%Henrik

%clc;
clear
%close all

%% Define Physical Parameters
pp.denw = 1000;     %density of water
pp.grav = 9.81;     %gravitational acceleration
pp.velc = [0 0 0]'; %constant irrotational current velocity, inertial frame
pp.gdir = [0 0 -1]';%direction of gravity in inertial frame

%% Adding switch to put on weight when base link reaches target.
pp.switch_weight = 0;

%% Define Snake Parameters
snake.link_vec  = [8 8 8 8 8 8]; %The type of links of the snake
snake.joint_vec = [3 3 3 3 3];   %The type of joints of the snake      
snake.n         = length(snake.link_vec);

C_a_C   =   1; % Added mass coefficient for the cross section, 
               %   1 is the theoretical inviscid result
C_d_1   =  .1; % nonlinear drag coefficient in surge
C_d_4   =  .1; % nonlinear drag coefficient in the roll direction for 
               %   the cross-section
C_d_C   =  .8; % nonlinear crossflow drag coefficient
C_d_L   =  .1; % Linear cross-sectional drag coefficient
alpha   =  .1; % Added mass ratio in surge/heave for a link
beta    =  .1; % Linear drag parameter in surge
gamma   =  .1; % Linear drag parameter in roll

masscoeff = [C_a_C, alpha];
dragcoeff = [C_d_1, C_d_4, C_d_C, C_d_L, beta, gamma];
clear C_a_C C_d_C C_d_1 C_d_4 C_d_L alpha beta gamma

%% Define Link Properties
link = define_link_properties_eely(pp, masscoeff, dragcoeff);

%% Define Joint Properties
joint = define_joint_properties();
                  
%% Define Simulation Parameters
Tmax = 1;
h = 0.001;                   %RK4 step size
deltaT_interp = 0.1;         %Time step size for the interpolated solution
deltaT_plot = 0.1;           %Time step size for the plotted solution

x_0 = zeros(26+3*snake.n,1); %Initial conditions
x_0(4) = 1;                  %"zero" quaternion
x_0(8:6+snake.n) = 0;        %Initial angle of joints

%% Build Snake
snake = build_snake(snake, link, joint, pp);

%% Simulate
output = RK4solver(h,Tmax,x_0,snake,pp);
T_log = output.T_log;
x_log = output.x_log;

%% Plot results
% run plotResults.m;
%% Show Movie
% run showMovie.m;



