function x_dot = dyn(t,x,snake,pp,flag,tau)

startTime = tic;

%Get states
pos       =  x(1:3);
quat      =  x(4:7);
theta     =  x(8 : 6+snake.n); %Angles of all joints
zeta      =  x(7+snake.n : 11+2*snake.n);
%Parameters from updating the factor, gamma, in the adaptive 
%control law for the whole body and for links.
gamma_hat = x(11+2*snake.n+1 : 11+2*snake.n+5);
gamma_hat_link = x(11+2*snake.n+6 : 11+2*snake.n+10);
eta_d = x(12+2*snake.n+10 : 17+2*snake.n + 10);
theta_d = x(18+2*snake.n+10 : 16+3*snake.n + 10);

%Compute rotation matrices and jacobians
[R, J]    =  comp_rot_jac(quat,theta,snake);
%Compute mass matrix
M         =  comp_mass(J,snake);

%Compute relative velocity to take into account the effect of current
velc_h    =  R(:,:,1)'*pp.velc;
nu        =  zeta(1:6);
zeta(1:3) =  zeta(1:3)-velc_h;

%Compute coriolis, drag, and hydrostatic terms with the relative velocity
[c, d, g] =  comp_cdg(J,R,theta,snake,pp,zeta);

%Compute the thrust-configuration matrix
B         =  comp_TCM(J,snake);

%Get position of snake base (tail)
eulerAngles = getEulerAngles(R(:,:,1));
eta = [pos; eulerAngles];

%End effector state
eta_ee = getEEpos(snake, pos, theta, R);

%****************************************************
%**********           Tasks              ************
%****************************************************

theta_dotd = zeta_ref(7:5+snake.n);

%Transforming to NED-frame
T = getTransformationMatrix(eulerAngles);
velocity_ned_d = R(:,:,1)*zeta_ref(1:3);
angvelocity_ned_d = T*zeta_ref(4:6);

%              Differentiating zeta_ref
%zeta_ref_dot = (zeta_ref - prev_zeta_ref)/(t-prev_t);
eta_dotdotd = zeros(6,1);
%eta_dotdotd = zeta_ref_dot(1:6);
theta_dotdotd = zeros(snake.n-1,1);
%theta_dotdotd = zeta_ref_dot(7:5+snake.n);



%_________________Thrust Allocation________________

%disp(length(B(1,:)));
%disp(length(B(:,1)));

thrust = zeros(snake.n_thruster_tot,1);
%thrust = pinv(B)*[tau_d;zeros(snake.n - 1,1)];
%thrust = pinv(B)*[tau_d;joint_torque];


disp(['Elapsed time: ' num2str(t)])

%Compute the relative acceleration
zeta_dot      =  M\(B*thrust+[tau_d;joint_torque]-c-d-g); %tau = B*thrust, tau_d;joint_torque
%Remove the effect of current
zeta_dot(1:3) = zeta_dot(1:3)-cross(nu(4:6),velc_h);
%Compute the time derivatives of position and orientation
pos_dot   =  R(:,:,1)*nu(1:3);  
quat_dot  =  TransAngQ(quat)*nu(4:6)+1*(1-quat'*quat)*quat; 
%Assemble derivative of the state vector
x_dot = [pos_dot ; quat_dot ; zeta(7:snake.n+5) ; zeta_dot ; gamma_hat_dot ; gamma_hat_dot_link; velocity_ned_d; angvelocity_ned_d; theta_dotd];

nin  = nargin;
if nin == 5 && flag ==1
    x_dot = [thrust; joint_torque];
end
stopTime = toc(startTime);
%fprintf('Total time: %d  \n', stopTime)

end
