function x_dot = dyn(t,x,snake,pp,flag)

startTime = tic;

%Get states
pos       =  x(1:3);
quat      =  x(4:7);
theta     =  x(8 : 6+snake.n); %Angles of all joints
zeta      =  x(7+snake.n : 11+2*snake.n);
%Parameters from updating the factor, gamma, in the adaptive 
%control law for the whole body and for links.
gamma_hat = x(11+2*snake.n+1 : 11+2*snake.n+5);
gamma_hat_link = x(11+2*snake.n+6 : 11+2*snake.n+10);
eta_d = x(12+2*snake.n+10 : 17+2*snake.n + 10);
theta_d = x(18+2*snake.n+10 : 16+3*snake.n + 10);

%Compute rotation matrices and jacobians
[R, J]    =  comp_rot_jac(quat,theta,snake);
%Compute mass matrix
M         =  comp_mass(J,snake);

%Compute relative velocity to take into account the effect of current
velc_h    =  R(:,:,1)'*pp.velc;
nu        =  zeta(1:6);
zeta(1:3) =  zeta(1:3)-velc_h;

%Compute coriolis, drag, and hydrostatic terms with the relative velocity
[c, d, g] =  comp_cdg(J,R,theta,snake,pp,zeta);

%Compute the thrust-configuration matrix
B         =  comp_TCM(J,snake);

%Get speed and position of snake base (tail)
eulerAngles = getEulerAngles(R(:,:,1));
eta = [pos; eulerAngles];

%End effector state
eta_ee = getEEpos(snake, pos, theta, R);

%****************************************************
%**********           Tasks              ************
%****************************************************

%              Position of end effector       
eta_ee_d = [4 0 0 0 0 0]';
eta_ee_d_dot = [0 0 0 0 0 0]';
K_ee = 0.5;
w_ee = eta_ee_d_dot + K_ee*(eta_ee_d - eta_ee);
J_ee = J(:,:,snake.n);

%              Position of base link
eta_base_d = [0 0 0 0 0 0]';
eta_base_d_dot = [0 0 0 0 0 0]';
K_base = 0.5;
w_base = eta_base_d_dot + K_base*(eta_base_d - eta);
J_base = J(:,:,1);

%              Inverse Kinematics
%zeta_ref = zeros(5+snake.n,1);

%zeta_ref = pinv(J_base)*w_base + (eye - pinv(J_base)*J_base)*pinv(J_ee)*w_ee;
zeta_ref = pinv(J_ee)*w_ee + (eye - pinv(J_ee)*J_ee)*pinv(J_base)*w_base;
%zeta_ref = J_ee'*w_ee + (eye(5+snake.n) - J_ee'*J_ee)*J_base'*w_base;

theta_dotd = zeta_ref(7:5+snake.n);

%Transforming to NED-frame
T = getTransformationMatrix(eulerAngles);
velocity_ned_d = R(:,:,1)*zeta_ref(1:3);
angvelocity_ned_d = T*zeta_ref(4:6);

%              Differentiating zeta_ref
%zeta_ref_dot = (zeta_ref - prev_zeta_ref)/(t-prev_t);
eta_dotdotd = zeros(6,1);
%eta_dotdotd = zeta_ref_dot(1:6);
theta_dotdotd = zeros(snake.n-1,1);
%theta_dotdotd = zeta_ref_dot(7:5+snake.n);


% %________Desired motion of whole snake_____________
%     
%     eta_d = zeros(6,1);
%     eta_dotd = zeros(6,1); 
%     eta_dotdotd = zeros(6,1);
%     
% %testing with a swimming gait for links:
% %__________________________________________________
%     phaseramp = .4*pi;
%     phase  = linspace(0,(snake.n-2)*phaseramp,snake.n-1)';
%     omega  = .8;
%     amp    = pi/4;   
%     
%     theta_d     = amp*sin(omega*t-phase);
%     theta_dotd = amp*omega*cos(omega*t-phase);
%     theta_dotdotd = -amp*omega*omega*sin(omega*t-phase);
%     thrust     = zeros(snake.n_thruster_tot,1);
%     
% %Testing with "stiff body" - controlling joints to zero angle
%     theta_d = zeros(length(snake.joint_vec),1);
%     theta_dotd = zeros(length(snake.joint_vec),1);
%     theta_dotdotd = zeros(length(snake.joint_vec),1);
%     
% %Testing with "U_shape"
%     angles = zeros(snake.n-1,1);
%     angles(1:(snake.n-1), 1) = pi/(snake.n-1);
%     theta_d = angles;
%     theta_dotd = zeros(snake.n-1,1);
%     theta_dotdotd = zeros(snake.n-1,1);
%  


    
%****************************************************
%*****    Non-regressor based adaptive control   ****
%****************************************************
%              For whole snake

    sigma = 5;
    f = ones(5,1);
    k = ones(6,1);
    
    eta_tilde = eta_d - eta;
    eta_dot = nu;
    eta_tilde_dot = [velocity_ned_d; angvelocity_ned_d] - eta_dot;
    
    s = eta_tilde_dot + sigma*eta_tilde;
    
    phi_1 = eta_dotdotd;
    phi_2 = eta_dot;
    phi_3 = k;
    phi_4 = eta_tilde_dot;
    phi_5 = eta_tilde;
    
    K_1 = gamma_hat(1)*s*phi_1'/max((norm(s)*norm(phi_1)),1);
    K_2 = gamma_hat(2)*s*phi_2'/max((norm(s)*norm(phi_2)),1);
    K_3 = gamma_hat(3)*s*phi_3'/max((norm(s)*norm(phi_3)),1);
    K_4 = gamma_hat(4)*s*phi_4'/max((norm(s)*norm(phi_4)),1);
    K_5 = gamma_hat(5)*s*phi_5'/max((norm(s)*norm(phi_5)),1);
    
    tau_d = K_1*phi_1 + K_2*phi_2 + K_3*phi_3 + K_4*phi_4 + K_5*phi_5;
    
    gamma_hat_dot = zeros(5,1);
    gamma_hat_dot(1) = f(1)*norm(s)*norm(phi_1);
    gamma_hat_dot(2) = f(2)*norm(s)*norm(phi_2);
    gamma_hat_dot(3) = f(3)*norm(s)*norm(phi_3);
    gamma_hat_dot(4) = f(4)*norm(s)*norm(phi_4);
    gamma_hat_dot(5) = f(5)*norm(s)*norm(phi_5);

%              For links
    sigma_link = 3;                      %Tunable, must be larger than zero. 
    f_link = ones(5,1);                  %Tunable, with each entry larger than zero.
    k_link = ones(length(snake.joint_vec),1); %Positive constant vector.
    
    theta_tilde = theta_d - theta;
    theta_tilde_dot = theta_dotd - zeta(7:snake.n+5);
    
    s_link = theta_tilde_dot + sigma_link*theta_tilde;
    
    phi_1_link = theta_dotdotd;
    phi_2_link = zeta(7:snake.n+5); %(= theta_dot)
    phi_3_link = k_link;
    phi_4_link = theta_tilde_dot;
    phi_5_link = theta_tilde;
    
    K_1_link = gamma_hat_link(1)*s_link*phi_1_link'/max((norm(s_link)*norm(phi_1_link)),1);
    K_2_link = gamma_hat_link(2)*s_link*phi_2_link'/max((norm(s_link)*norm(phi_2_link)),1);
    K_3_link = gamma_hat_link(3)*s_link*phi_3_link'/max((norm(s_link)*norm(phi_3_link)),1);
    K_4_link = gamma_hat_link(4)*s_link*phi_4_link'/max((norm(s_link)*norm(phi_4_link)),1);
    K_5_link = gamma_hat_link(5)*s_link*phi_5_link'/max((norm(s_link)*norm(phi_5_link)),1);
    
    joint_torque = K_1_link*phi_1_link + K_2_link*phi_2_link + K_3_link*phi_3_link + K_4_link*phi_4_link + K_5_link*phi_5_link;
    
    %Adaptive update
    gamma_hat_dot_link = zeros(5,1);
    gamma_hat_dot_link(1) = f_link(1)*norm(s_link)*norm(phi_1_link);
    gamma_hat_dot_link(2) = f_link(2)*norm(s_link)*norm(phi_2_link);
    gamma_hat_dot_link(3) = f_link(3)*norm(s_link)*norm(phi_3_link);
    gamma_hat_dot_link(4) = f_link(4)*norm(s_link)*norm(phi_4_link);
    gamma_hat_dot_link(5) = f_link(5)*norm(s_link)*norm(phi_5_link);
    
% __________________PD-control_____________________
   % Kp = 10;
   % Kd = 10;
    %joint_torque = (- Kp*(theta-theta_d)- Kd*(zeta(7:snake.n+5)-theta_dotd));
%__________________________________________________


%_________________Thrust Allocation________________

%disp(length(B(1,:)));
%disp(length(B(:,1)));

thrust = zeros(snake.n_thruster_tot,1);
%thrust = pinv(B)*[tau_d;zeros(snake.n - 1,1)];
%thrust = pinv(B)*[tau_d;joint_torque];


disp(['Elapsed time: ' num2str(t)])

%Compute the relative acceleration
zeta_dot      =  M\(B*thrust+[tau_d;joint_torque]-c-d-g); %tau = B*thrust, tau_d;joint_torque
%Remove the effect of current
zeta_dot(1:3) = zeta_dot(1:3)-cross(nu(4:6),velc_h);
%Compute the time derivatives of position and orientation
pos_dot   =  R(:,:,1)*nu(1:3);  
quat_dot  =  TransAngQ(quat)*nu(4:6)+1*(1-quat'*quat)*quat; 
%Assemble derivative of the state vector
x_dot = [pos_dot ; quat_dot ; zeta(7:snake.n+5) ; zeta_dot ; gamma_hat_dot ; gamma_hat_dot_link; velocity_ned_d; angvelocity_ned_d; theta_dotd];

nin  = nargin;
if nin == 5 && flag ==1
    x_dot = [thrust; joint_torque];
end
stopTime = toc(startTime);
%fprintf('Total time: %d  \n', stopTime)

end
