function x_dot = f_dyn(x,thrust,tau_d,joint_torque,snake,pp,update_param)

quat  = x(4:7);
theta = x(8 : 6+snake.n);
zeta  = x(7+snake.n : 11+2*snake.n);

%Compute rotation matrices and jacobians
[R, J]    =  comp_rot_jac(quat,theta,snake);

%Compute mass matrix
M         =  comp_mass(J,snake);

%Compute relative velocity to take into account the effect of current
velc_h    =  R(:,:,1)'*pp.velc;
nu        =  zeta(1:6);
zeta(1:3) =  zeta(1:3)-velc_h;

%Compute coriolis, drag, and hydrostatic terms with the relative velocity
[c, d, g] =  comp_cdg(J,R,theta,snake,pp,zeta);
%Compute the thrust-configuration matrix
B         =  comp_TCM(J,snake);

% Add weight at start of base link if switch is turned on
if (pp.switch_weight == 1)
    weightEE = [0 0 60 0 0 0]';
else
    weightEE = zeros(6,1);
end

%Compute the relative acceleration
zeta_dot      =  M\(B*thrust+[tau_d;joint_torque]-c-d-g-...
    [weightEE;zeros(snake.n-1,1)]);%

%Remove the effect of current
zeta_dot(1:3) = zeta_dot(1:3)-cross(nu(4:6),velc_h);
%Compute the time derivatives of position and orientation
pos_dot   =  R(:,:,1)*nu(1:3);  
quat_dot  =  TransAngQ(quat)*nu(4:6)+1*(1-quat'*quat)*quat; 
%Assemble derivative of the state vector
x_dot = [pos_dot ; quat_dot ; zeta(7:snake.n+5) ; zeta_dot ;...
    update_param.gamma_hat_dot ; update_param.gamma_hat_dot_link; ...
    update_param.velocity_ine_d; update_param.angvelocity_ine_d; ...
    update_param.theta_dotd];

end

