% This script uses the output from simulation to plot the interesting
% states, references, control output etc.

%% Taking the data out of struct format to simplify plotting
x_log = output.x_log;
tau_d_log = output.tau_d_log;
joint_torque_log = output.joint_torque_log;
x_ref_log = output.x_ref_log;
orientation_log = output.orientation_log;
T_log = output.T_log;
eta_ee_log = output.eta_ee_log;
eta_ee_ref_log = output.eta_ee_ref_log;
base_task = output.eta_base_task_log;
nominal_task = output.nominal_task_log;
theta_dotd_log = output.theta_dotd_log;
joint_controller_log = output.joint_controller_log;
phi_1_joint = joint_controller_log.phi_1;
phi_2_joint = joint_controller_log.phi_2;
phi_3_joint = joint_controller_log.phi_3;
phi_4_joint = joint_controller_log.phi_4;
phi_5_joint = joint_controller_log.phi_5;


%% Plot position of base link
figure(10)
subplot(3,1,1);
plot(T_log,x_log(1,:),'color','b');
hold on
plot(T_log,x_ref_log(1,:),'color','r');
hold on
plot(T_log,base_task(1,:),'color','c');
grid on
legend('\eta_1 (1)','\eta_{1,d} (1)', 'Task desired');
xlabel('t [s]');
ylabel('[m]');
title('X position of base link');

subplot(3,1,2);
plot(T_log,x_log(2,:),'color','b');
hold on
plot(T_log,x_ref_log(2,:),'color','r');
hold on
plot(T_log,base_task(2,:),'color','c');
grid on
legend('\eta_1 (2)','\eta_{1,d} (2)', 'Task desired');
xlabel('t [s]');
ylabel('[m]');
title('Y position of base link');

subplot(3,1,3);
plot(T_log,x_log(3,:),'color','b');
hold on
plot(T_log,x_ref_log(3,:),'color','r');
hold on
plot(T_log,base_task(3,:),'color','c');
grid on
legend('\eta_1 (3)','\eta_{1,d} (3)', 'Task desired');
xlabel('t [s]');
ylabel('[m]');
title('Z position of base link');

%% Plot orientation of base link
figure(20)
subplot(3,1,1);
plot(T_log,orientation_log(1,:),'color','b');
hold on
plot(T_log,x_ref_log(4,:),'color','r');
hold on
plot(T_log,base_task(4,:),'color','c');
grid on
legend('\phi','\phi_d', '\phi_{d,task}');
xlabel('t [s]');
ylabel('[rad]');
title('Roll angle of base link');

subplot(3,1,2);
plot(T_log,orientation_log(2,:),'color','b');
hold on
plot(T_log,x_ref_log(5,:),'color','r');
hold on
plot(T_log,base_task(5,:),'color','c');
grid on
legend('\theta','\theta_d', '\theta_{d,task}');
xlabel('t [s]');
ylabel('[rad]');
title('Pitch angle of base link');

subplot(3,1,3);
plot(T_log,orientation_log(3,:),'color','b');
hold on
plot(T_log,x_ref_log(6,:),'color','r');
hold on
plot(T_log,base_task(6,:),'color','c');
grid on
legend('\psi','\psi_d', '\psi_{d,task}');
xlabel('t [s]');
ylabel('[rad]');
title('Yaw angle of base link');

%% Plot position of end effector
figure(50)
subplot(3,1,1);
plot(T_log,eta_ee_log(1,:),'color','b');
hold on
plot(T_log,eta_ee_ref_log(1,:),'color','c');
grid on
legend('x_{ee}', 'x_{ee,d,task}');
xlabel('t [s]');
ylabel('[m]');
title('X position of end effector');

subplot(3,1,2);
plot(T_log,eta_ee_log(2,:),'color','b');
hold on
plot(T_log,eta_ee_ref_log(2,:),'color','c');
grid on
legend('y_{ee}', 'y_{ee,d,task}');
xlabel('t [s]');
ylabel('[m]');
title('Y position of end effector');

subplot(3,1,3);
plot(T_log,eta_ee_log(3,:),'color','b');
hold on
plot(T_log,eta_ee_ref_log(3,:),'color','c');
grid on
legend('z_{ee}', 'z_{ee,d,task}');
xlabel('t [s]');
ylabel('[m]');
title('Z position of end effector');

%% Plot orientation of end effector
figure(60)
subplot(3,1,1);
plot(T_log,eta_ee_log(4,:),'color','b');
hold on
plot(T_log,eta_ee_ref_log(4,:),'color','c');
grid on
legend('\phi_{ee}', '\phi_{ee,d,task}');
xlabel('t [s]');
ylabel('[rad]');
title('Roll angle of end effector');

subplot(3,1,2);
plot(T_log,eta_ee_log(5,:),'color','b');
hold on
plot(T_log,eta_ee_ref_log(5,:),'color','c');
grid on
legend('\theta_{ee}', '\theta_{ee,d,task}');
xlabel('t [s]');
ylabel('[rad]');
title('Pitch angle of end effector');

subplot(3,1,3);
plot(T_log,eta_ee_log(6,:),'color','b');
hold on
plot(T_log,eta_ee_ref_log(6,:),'color','c');
grid on
legend('\psi_{ee}', '\psi_{ee,d,task}');
xlabel('t [s]');
ylabel('[rad]');
title('Yaw angle of end effector');

%% Plot joint angles
%Plot joint angles

figure(100)
n_joint = snake.n-1;
for i=1 : n_joint
    %figure(100+i)
    subplot(n_joint,1,i);
    plot(T_log,x_log(7+i,:),'color','b');
    hold on
    plot(T_log,x_ref_log(6+i,:),'color','r');
%     hold on
%     plot(T_log,nominal_task(i,:),'color','c');
    grid on
    legend(['\theta_{' num2str(i) '}'],['\theta_{' num2str(i)...
        ',d}']);%, ['\theta_{' num2str(i) ',task}']);
    xlabel('t [s]');
    ylabel('[rad]');
    title(['Angle of joint ' num2str(i)]);
end

%% Plot translational forces on base link from control law
figure(210)
subplot(3,1,1);
plot(T_log,tau_d_log(1,:),'color','b');
grid on
legend('\tau (1)');
xlabel('t [s]');
ylabel('[N]');
title('Thrust in surge from base link controller');

subplot(3,1,2);
plot(T_log,tau_d_log(2,:),'color','b');
grid on
legend('\tau (2)');
xlabel('t [s]');
ylabel('[N]');
title('Thrust in sway from base link controller');

subplot(3,1,3);
plot(T_log,tau_d_log(3,:),'color','b');
grid on
legend('\tau (3)');
xlabel('t [s]');
ylabel('[N]');
title('Thrust in heave from base link controller');

%% Plot moments applied by the control law
figure(211)
subplot(3,1,1);
plot(T_log,tau_d_log(4,:),'color','b');
grid on
legend('\tau (4)');
xlabel('t [s]');
ylabel('[Nm]');
title('Moment in roll from base link controller');

subplot(3,1,2);
plot(T_log,tau_d_log(5,:),'color','b');
grid on
legend('\tau (5)');
xlabel('t [s]');
ylabel('[Nm]');
title('Moment in pitch from base link controller');

subplot(3,1,3);
plot(T_log,tau_d_log(6,:),'color','b');
grid on
legend('\tau (6)');
xlabel('t [s]');
ylabel('[Nm]');
title('Moment in yaw from base link controller');

%% Plot tau for forces on joints
%Forces on joints
figure(400)
for i=1 : snake.n-1
    subplot(snake.n-1,1,i);
    plot(T_log,joint_torque_log(i,:),'color','b');
    grid on
    legend(['\tau_{joint} (' num2str(i) ')']);
    xlabel('t [s]');
    ylabel('[Nm]');
    title(['Torque on joint ' num2str(i)]);
end


%% Plot joint velocities
figure(600)
for i = 1:snake.n-1
    subplot(snake.n-1,1,i);
    plot(T_log,x_log(12+snake.n+i,:),'color','b');
    hold on
    plot(T_log,x_ref_log(11+snake.n+i,:),'color','m');
    grid on
    legend(['Velocitiy of joint  ' num2str(i)],'Desired velocity');
    xlabel('t [s]');
    ylabel('[rad/s]');
    title(['Velocity of joint  ' num2str(i)]);
end

